# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: user-admin.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from . import types_pb2 as types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10user-admin.proto\x12\x10\x61\x65rospike.vector\x1a\x1bgoogle/protobuf/empty.proto\x1a\x0btypes.proto\"S\n\x0e\x41\x64\x64UserRequest\x12\x32\n\x0b\x63redentials\x18\x01 \x01(\x0b\x32\x1d.aerospike.vector.Credentials\x12\r\n\x05roles\x18\x02 \x03(\t\"N\n\x18UpdateCredentialsRequest\x12\x32\n\x0b\x63redentials\x18\x01 \x01(\x0b\x32\x1d.aerospike.vector.Credentials\"#\n\x0f\x44ropUserRequest\x12\x10\n\x08username\x18\x01 \x01(\t\"\"\n\x0eGetUserRequest\x12\x10\n\x08username\x18\x01 \x01(\t\"4\n\x11GrantRolesRequest\x12\x10\n\x08username\x18\x01 \x01(\t\x12\r\n\x05roles\x18\x02 \x03(\t\"5\n\x12RevokeRolesRequest\x12\x10\n\x08username\x18\x01 \x01(\t\x12\r\n\x05roles\x18\x02 \x03(\t\":\n\x11ListRolesResponse\x12%\n\x05roles\x18\x01 \x03(\x0b\x32\x16.aerospike.vector.Role\":\n\x11ListUsersResponse\x12%\n\x05users\x18\x01 \x03(\x0b\x32\x16.aerospike.vector.User2\xf8\x04\n\x10UserAdminService\x12\x45\n\x07\x41\x64\x64User\x12 .aerospike.vector.AddUserRequest\x1a\x16.google.protobuf.Empty\"\x00\x12Y\n\x11UpdateCredentials\x12*.aerospike.vector.UpdateCredentialsRequest\x1a\x16.google.protobuf.Empty\"\x00\x12G\n\x08\x44ropUser\x12!.aerospike.vector.DropUserRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x45\n\x07GetUser\x12 .aerospike.vector.GetUserRequest\x1a\x16.aerospike.vector.User\"\x00\x12J\n\tListUsers\x12\x16.google.protobuf.Empty\x1a#.aerospike.vector.ListUsersResponse\"\x00\x12K\n\nGrantRoles\x12#.aerospike.vector.GrantRolesRequest\x1a\x16.google.protobuf.Empty\"\x00\x12M\n\x0bRevokeRoles\x12$.aerospike.vector.RevokeRolesRequest\x1a\x16.google.protobuf.Empty\"\x00\x12J\n\tListRoles\x12\x16.google.protobuf.Empty\x1a#.aerospike.vector.ListRolesResponse\"\x00\x42\x43\n!com.aerospike.vector.client.protoP\x01Z\x1c\x61\x65rospike.com/vector/protos/b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'user_admin_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n!com.aerospike.vector.client.protoP\001Z\034aerospike.com/vector/protos/'
  _globals['_ADDUSERREQUEST']._serialized_start=80
  _globals['_ADDUSERREQUEST']._serialized_end=163
  _globals['_UPDATECREDENTIALSREQUEST']._serialized_start=165
  _globals['_UPDATECREDENTIALSREQUEST']._serialized_end=243
  _globals['_DROPUSERREQUEST']._serialized_start=245
  _globals['_DROPUSERREQUEST']._serialized_end=280
  _globals['_GETUSERREQUEST']._serialized_start=282
  _globals['_GETUSERREQUEST']._serialized_end=316
  _globals['_GRANTROLESREQUEST']._serialized_start=318
  _globals['_GRANTROLESREQUEST']._serialized_end=370
  _globals['_REVOKEROLESREQUEST']._serialized_start=372
  _globals['_REVOKEROLESREQUEST']._serialized_end=425
  _globals['_LISTROLESRESPONSE']._serialized_start=427
  _globals['_LISTROLESRESPONSE']._serialized_end=485
  _globals['_LISTUSERSRESPONSE']._serialized_start=487
  _globals['_LISTUSERSRESPONSE']._serialized_end=545
  _globals['_USERADMINSERVICE']._serialized_start=548
  _globals['_USERADMINSERVICE']._serialized_end=1180
# @@protoc_insertion_point(module_scope)
