# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: index.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from . import types_pb2 as types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bindex.proto\x12\x10\x61\x65rospike.vector\x1a\x1bgoogle/protobuf/empty.proto\x1a\x0btypes.proto\"2\n\x13IndexStatusResponse\x12\x1b\n\x13unmergedRecordCount\x18\x02 \x01(\x03\"_\n\x18GcInvalidVerticesRequest\x12*\n\x07indexId\x18\x01 \x01(\x0b\x32\x19.aerospike.vector.IndexId\x12\x17\n\x0f\x63utoffTimestamp\x18\x02 \x01(\x03\"\xf9\x01\n\x12IndexUpdateRequest\x12*\n\x07indexId\x18\x01 \x01(\x0b\x32\x19.aerospike.vector.IndexId\x12@\n\x06labels\x18\x02 \x03(\x0b\x32\x30.aerospike.vector.IndexUpdateRequest.LabelsEntry\x12<\n\x0fhnswIndexUpdate\x18\x03 \x01(\x0b\x32!.aerospike.vector.HnswIndexUpdateH\x00\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x08\n\x06update2\x98\x04\n\x0cIndexService\x12\x45\n\x06\x43reate\x12!.aerospike.vector.IndexDefinition\x1a\x16.google.protobuf.Empty\"\x00\x12H\n\x06Update\x12$.aerospike.vector.IndexUpdateRequest\x1a\x16.google.protobuf.Empty\"\x00\x12;\n\x04\x44rop\x12\x19.aerospike.vector.IndexId\x1a\x16.google.protobuf.Empty\"\x00\x12G\n\x04List\x12\x16.google.protobuf.Empty\x1a%.aerospike.vector.IndexDefinitionList\"\x00\x12\x45\n\x03Get\x12\x19.aerospike.vector.IndexId\x1a!.aerospike.vector.IndexDefinition\"\x00\x12O\n\tGetStatus\x12\x19.aerospike.vector.IndexId\x1a%.aerospike.vector.IndexStatusResponse\"\x00\x12Y\n\x11GcInvalidVertices\x12*.aerospike.vector.GcInvalidVerticesRequest\x1a\x16.google.protobuf.Empty\"\x00\x42\x43\n!com.aerospike.vector.client.protoP\x01Z\x1c\x61\x65rospike.com/vector/protos/b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'index_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n!com.aerospike.vector.client.protoP\001Z\034aerospike.com/vector/protos/'
  _globals['_INDEXUPDATEREQUEST_LABELSENTRY']._loaded_options = None
  _globals['_INDEXUPDATEREQUEST_LABELSENTRY']._serialized_options = b'8\001'
  _globals['_INDEXSTATUSRESPONSE']._serialized_start=75
  _globals['_INDEXSTATUSRESPONSE']._serialized_end=125
  _globals['_GCINVALIDVERTICESREQUEST']._serialized_start=127
  _globals['_GCINVALIDVERTICESREQUEST']._serialized_end=222
  _globals['_INDEXUPDATEREQUEST']._serialized_start=225
  _globals['_INDEXUPDATEREQUEST']._serialized_end=474
  _globals['_INDEXUPDATEREQUEST_LABELSENTRY']._serialized_start=419
  _globals['_INDEXUPDATEREQUEST_LABELSENTRY']._serialized_end=464
  _globals['_INDEXSERVICE']._serialized_start=477
  _globals['_INDEXSERVICE']._serialized_end=1013
# @@protoc_insertion_point(module_scope)
