# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vector-db.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fvector-db.proto\x12\x10\x61\x65rospike.vector\x1a\x1bgoogle/protobuf/empty.proto\"\x0e\n\x0c\x41\x62outRequest\" \n\rAboutResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\"\x17\n\tClusterId\x12\n\n\x02id\x18\x01 \x01(\x04\"\x14\n\x06NodeId\x12\n\n\x02id\x18\x01 \x01(\x04\">\n\x0eServerEndpoint\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\r\x12\r\n\x05isTls\x18\x03 \x01(\x08\"I\n\x12ServerEndpointList\x12\x33\n\tendpoints\x18\x01 \x03(\x0b\x32 .aerospike.vector.ServerEndpoint\"\xb8\x01\n\x14\x43lusterNodeEndpoints\x12H\n\tendpoints\x18\x01 \x03(\x0b\x32\x35.aerospike.vector.ClusterNodeEndpoints.EndpointsEntry\x1aV\n\x0e\x45ndpointsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x04\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32$.aerospike.vector.ServerEndpointList:\x02\x38\x01\"I\n\x1b\x43lusterNodeEndpointsRequest\x12\x19\n\x0clistenerName\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_listenerName\"\x81\x01\n\x0f\x43lusteringState\x12\x13\n\x0bisInCluster\x18\x01 \x01(\x08\x12.\n\tclusterId\x18\x02 \x01(\x0b\x32\x1b.aerospike.vector.ClusterId\x12)\n\x07members\x18\x03 \x03(\x0b\x32\x18.aerospike.vector.NodeId2X\n\x0c\x41\x62outService\x12H\n\x03Get\x12\x1e.aerospike.vector.AboutRequest\x1a\x1f.aerospike.vector.AboutResponse\"\x00\x32\xdf\x02\n\x12\x43lusterInfoService\x12?\n\tGetNodeId\x12\x16.google.protobuf.Empty\x1a\x18.aerospike.vector.NodeId\"\x00\x12\x45\n\x0cGetClusterId\x12\x16.google.protobuf.Empty\x1a\x1b.aerospike.vector.ClusterId\"\x00\x12Q\n\x12GetClusteringState\x12\x16.google.protobuf.Empty\x1a!.aerospike.vector.ClusteringState\"\x00\x12n\n\x13GetClusterEndpoints\x12-.aerospike.vector.ClusterNodeEndpointsRequest\x1a&.aerospike.vector.ClusterNodeEndpoints\"\x00\x42\x43\n!com.aerospike.vector.client.protoP\x01Z\x1c\x61\x65rospike.com/vector/protos/b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'vector_db_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n!com.aerospike.vector.client.protoP\001Z\034aerospike.com/vector/protos/'
  _globals['_CLUSTERNODEENDPOINTS_ENDPOINTSENTRY']._loaded_options = None
  _globals['_CLUSTERNODEENDPOINTS_ENDPOINTSENTRY']._serialized_options = b'8\001'
  _globals['_ABOUTREQUEST']._serialized_start=66
  _globals['_ABOUTREQUEST']._serialized_end=80
  _globals['_ABOUTRESPONSE']._serialized_start=82
  _globals['_ABOUTRESPONSE']._serialized_end=114
  _globals['_CLUSTERID']._serialized_start=116
  _globals['_CLUSTERID']._serialized_end=139
  _globals['_NODEID']._serialized_start=141
  _globals['_NODEID']._serialized_end=161
  _globals['_SERVERENDPOINT']._serialized_start=163
  _globals['_SERVERENDPOINT']._serialized_end=225
  _globals['_SERVERENDPOINTLIST']._serialized_start=227
  _globals['_SERVERENDPOINTLIST']._serialized_end=300
  _globals['_CLUSTERNODEENDPOINTS']._serialized_start=303
  _globals['_CLUSTERNODEENDPOINTS']._serialized_end=487
  _globals['_CLUSTERNODEENDPOINTS_ENDPOINTSENTRY']._serialized_start=401
  _globals['_CLUSTERNODEENDPOINTS_ENDPOINTSENTRY']._serialized_end=487
  _globals['_CLUSTERNODEENDPOINTSREQUEST']._serialized_start=489
  _globals['_CLUSTERNODEENDPOINTSREQUEST']._serialized_end=562
  _globals['_CLUSTERINGSTATE']._serialized_start=565
  _globals['_CLUSTERINGSTATE']._serialized_end=694
  _globals['_ABOUTSERVICE']._serialized_start=696
  _globals['_ABOUTSERVICE']._serialized_end=784
  _globals['_CLUSTERINFOSERVICE']._serialized_start=787
  _globals['_CLUSTERINFOSERVICE']._serialized_end=1138
# @@protoc_insertion_point(module_scope)
